"""Tests for the expression class hierarchy (composite and decorator patterns)."""

from therismos.expr import (
    FALSE,
    TRUE,
    AllExpr,
    AnyExpr,
    AtomicExpr,
    ComparisonExpr,
    CompositeExpr,
    DecoratorExpr,
    Eq,
    Expr,
    F,
    FieldBasedExpr,
    Gt,
    IsNull,
    MultiValuedFieldExpr,
    NotExpr,
    SingleValuedFieldExpr,
)


class TestAtomicExprInheritance:
    """Test that all atomic expressions inherit from AtomicExpr."""

    def test_eq_is_atomic(self) -> None:
        """Test that Eq is an atomic expression."""
        expr = F("x") == 5
        assert isinstance(expr, AtomicExpr)
        assert isinstance(expr, Expr)

    def test_ne_is_atomic(self) -> None:
        """Test that Ne is an atomic expression."""
        expr = F("x") != 5
        assert isinstance(expr, AtomicExpr)
        assert isinstance(expr, Expr)

    def test_lt_is_atomic(self) -> None:
        """Test that Lt is an atomic expression."""
        expr = F("x") < 5
        assert isinstance(expr, AtomicExpr)
        assert isinstance(expr, Expr)

    def test_le_is_atomic(self) -> None:
        """Test that Le is an atomic expression."""
        expr = F("x") <= 5
        assert isinstance(expr, AtomicExpr)
        assert isinstance(expr, Expr)

    def test_gt_is_atomic(self) -> None:
        """Test that Gt is an atomic expression."""
        expr = F("x") > 5
        assert isinstance(expr, AtomicExpr)
        assert isinstance(expr, Expr)

    def test_ge_is_atomic(self) -> None:
        """Test that Ge is an atomic expression."""
        expr = F("x") >= 5
        assert isinstance(expr, AtomicExpr)
        assert isinstance(expr, Expr)

    def test_regex_is_atomic(self) -> None:
        """Test that Regex is an atomic expression."""
        expr = F("name").matches(r"test.*")
        assert isinstance(expr, AtomicExpr)
        assert isinstance(expr, Expr)

    def test_in_is_atomic(self) -> None:
        """Test that In is an atomic expression."""
        expr = F("x").is_in(1, 2, 3)
        assert isinstance(expr, AtomicExpr)
        assert isinstance(expr, Expr)

    def test_is_null_is_atomic(self) -> None:
        """Test that IsNull is an atomic expression."""
        expr = F("x").is_null()
        assert isinstance(expr, AtomicExpr)
        assert isinstance(expr, Expr)

    def test_true_expr_is_atomic(self) -> None:
        """Test that TrueExpr is an atomic expression."""
        assert isinstance(TRUE, AtomicExpr)
        assert isinstance(TRUE, Expr)

    def test_false_expr_is_atomic(self) -> None:
        """Test that FalseExpr is an atomic expression."""
        assert isinstance(FALSE, AtomicExpr)
        assert isinstance(FALSE, Expr)


class TestCompositeExprInheritance:
    """Test that composite expressions inherit from CompositeExpr."""

    def test_all_expr_is_composite(self) -> None:
        """Test that AllExpr is a composite expression."""
        expr = AllExpr(F("x") == 1, F("y") == 2)
        assert isinstance(expr, CompositeExpr)
        assert isinstance(expr, Expr)

    def test_any_expr_is_composite(self) -> None:
        """Test that AnyExpr is a composite expression."""
        expr = AnyExpr(F("x") == 1, F("y") == 2)
        assert isinstance(expr, CompositeExpr)
        assert isinstance(expr, Expr)

    def test_all_expr_via_and_operator(self) -> None:
        """Test that & operator creates composite expressions."""
        expr = (F("x") == 1) & (F("y") == 2)
        assert isinstance(expr, CompositeExpr)
        assert isinstance(expr, AllExpr)

    def test_any_expr_via_or_operator(self) -> None:
        """Test that | operator creates composite expressions."""
        expr = (F("x") == 1) | (F("y") == 2)
        assert isinstance(expr, CompositeExpr)
        assert isinstance(expr, AnyExpr)


class TestDecoratorExprInheritance:
    """Test that decorator expressions inherit from DecoratorExpr."""

    def test_not_expr_is_decorator(self) -> None:
        """Test that NotExpr is a decorator expression."""
        expr = NotExpr(F("x") == 1)
        assert isinstance(expr, DecoratorExpr)
        assert isinstance(expr, Expr)

    def test_not_expr_via_invert_operator(self) -> None:
        """Test that ~ operator creates decorator expressions."""
        expr = ~(F("x") == 1)
        assert isinstance(expr, DecoratorExpr)
        assert isinstance(expr, NotExpr)


class TestClassHierarchyStructure:
    """Test the overall structure of the class hierarchy."""

    def test_atomic_composite_decorator_are_distinct(self) -> None:
        """Test that atomic, composite, and decorator are distinct categories."""
        atomic = F("x") == 1
        composite = AllExpr(F("x") == 1, F("y") == 2)
        decorator = NotExpr(F("x") == 1)

        # Atomic is not composite or decorator
        assert isinstance(atomic, AtomicExpr)
        assert not isinstance(atomic, CompositeExpr)
        assert not isinstance(atomic, DecoratorExpr)

        # Composite is not atomic or decorator
        assert not isinstance(composite, AtomicExpr)
        assert isinstance(composite, CompositeExpr)
        assert not isinstance(composite, DecoratorExpr)

        # Decorator is not atomic or composite
        assert not isinstance(decorator, AtomicExpr)
        assert not isinstance(decorator, CompositeExpr)
        assert isinstance(decorator, DecoratorExpr)

    def test_all_expressions_are_expr(self) -> None:
        """Test that all expression types inherit from Expr."""
        expressions = [
            F("x") == 1,  # Eq (atomic)
            F("x") != 1,  # Ne (atomic)
            F("x") < 1,  # Lt (atomic)
            F("x") <= 1,  # Le (atomic)
            F("x") > 1,  # Gt (atomic)
            F("x") >= 1,  # Ge (atomic)
            F("name").matches(r"test.*"),  # Regex (atomic)
            F("x").is_in(1, 2, 3),  # In (atomic)
            F("x").is_null(),  # IsNull (atomic)
            TRUE,  # TrueExpr (atomic)
            FALSE,  # FalseExpr (atomic)
            AllExpr(F("x") == 1, F("y") == 2),  # AllExpr (composite)
            AnyExpr(F("x") == 1, F("y") == 2),  # AnyExpr (composite)
            NotExpr(F("x") == 1),  # NotExpr (decorator)
        ]

        for expr in expressions:
            assert isinstance(expr, Expr)

    def test_complex_nested_expression(self) -> None:
        """Test a complex nested expression with all types."""
        # Build: NOT((x == 1 AND y > 2) OR z IS NULL)
        expr = NotExpr(AnyExpr(AllExpr(Eq(F("x"), 1), Gt(F("y"), 2)), IsNull(F("z"), True)))

        # Root is decorator
        assert isinstance(expr, DecoratorExpr)
        assert isinstance(expr, NotExpr)

        # First child is composite (AnyExpr)
        assert isinstance(expr.expr, CompositeExpr)
        assert isinstance(expr.expr, AnyExpr)

        # First child of AnyExpr is composite (AllExpr)
        assert isinstance(expr.expr.exprs[0], CompositeExpr)
        assert isinstance(expr.expr.exprs[0], AllExpr)

        # Children of AllExpr are atomic
        assert isinstance(expr.expr.exprs[0].exprs[0], AtomicExpr)  # Eq
        assert isinstance(expr.expr.exprs[0].exprs[1], AtomicExpr)  # Gt

        # Second child of AnyExpr is atomic
        assert isinstance(expr.expr.exprs[1], AtomicExpr)  # IsNull


class TestIntermediateClassInheritance:
    """Test the intermediate class hierarchy (FieldBasedExpr, etc.)."""

    def test_comparison_expressions_are_field_based(self) -> None:
        """Test that comparison expressions inherit from FieldBasedExpr."""
        expressions = [
            F("x") == 1,  # Eq
            F("x") != 1,  # Ne
            F("x") < 1,  # Lt
            F("x") <= 1,  # Le
            F("x") > 1,  # Gt
            F("x") >= 1,  # Ge
        ]
        for expr in expressions:
            assert isinstance(expr, FieldBasedExpr)
            assert isinstance(expr, AtomicExpr)

    def test_regex_is_field_based(self) -> None:
        """Test that Regex inherits from FieldBasedExpr."""
        expr = F("name").matches(r"test.*")
        assert isinstance(expr, FieldBasedExpr)
        assert isinstance(expr, AtomicExpr)

    def test_multi_valued_expressions_are_field_based(self) -> None:
        """Test that In and IsNull inherit from FieldBasedExpr."""
        in_expr = F("x").is_in(1, 2, 3)
        is_null_expr = F("x").is_null()

        assert isinstance(in_expr, FieldBasedExpr)
        assert isinstance(is_null_expr, FieldBasedExpr)

    def test_comparison_expressions_are_single_valued(self) -> None:
        """Test that comparison expressions inherit from SingleValuedFieldExpr."""
        expressions = [
            F("x") == 1,  # Eq
            F("x") != 1,  # Ne
            F("x") < 1,  # Lt
            F("x") <= 1,  # Le
            F("x") > 1,  # Gt
            F("x") >= 1,  # Ge
        ]
        for expr in expressions:
            assert isinstance(expr, SingleValuedFieldExpr)
            assert isinstance(expr, FieldBasedExpr)

    def test_regex_is_single_valued(self) -> None:
        """Test that Regex inherits from SingleValuedFieldExpr."""
        expr = F("name").matches(r"test.*")
        assert isinstance(expr, SingleValuedFieldExpr)
        assert isinstance(expr, FieldBasedExpr)

    def test_comparison_expressions_are_comparison_expr(self) -> None:
        """Test that comparison expressions inherit from ComparisonExpr."""
        expressions = [
            F("x") == 1,  # Eq
            F("x") != 1,  # Ne
            F("x") < 1,  # Lt
            F("x") <= 1,  # Le
            F("x") > 1,  # Gt
            F("x") >= 1,  # Ge
        ]
        for expr in expressions:
            assert isinstance(expr, ComparisonExpr)
            assert isinstance(expr, SingleValuedFieldExpr)

    def test_regex_is_not_comparison_expr(self) -> None:
        """Test that Regex does NOT inherit from ComparisonExpr."""
        expr = F("name").matches(r"test.*")
        assert not isinstance(expr, ComparisonExpr)
        assert isinstance(expr, SingleValuedFieldExpr)

    def test_multi_valued_expressions_are_multi_valued(self) -> None:
        """Test that In and IsNull inherit from MultiValuedFieldExpr."""
        in_expr = F("x").is_in(1, 2, 3)
        is_null_expr = F("x").is_null()

        assert isinstance(in_expr, MultiValuedFieldExpr)
        assert isinstance(is_null_expr, MultiValuedFieldExpr)
        assert isinstance(in_expr, FieldBasedExpr)
        assert isinstance(is_null_expr, FieldBasedExpr)

    def test_constants_are_not_field_based(self) -> None:
        """Test that TRUE and FALSE do NOT inherit from FieldBasedExpr."""
        assert not isinstance(TRUE, FieldBasedExpr)
        assert not isinstance(FALSE, FieldBasedExpr)
        assert isinstance(TRUE, AtomicExpr)
        assert isinstance(FALSE, AtomicExpr)


class TestBaseClassAbstractness:
    """Test that base classes are abstract and cannot be instantiated directly."""

    def test_expr_is_abstract(self) -> None:
        """Test that Expr cannot be instantiated directly."""
        try:
            Expr()  # type: ignore[abstract]
            raise AssertionError("Should not be able to instantiate Expr")
        except TypeError as e:
            assert "abstract" in str(e).lower()

    def test_atomic_expr_is_abstract(self) -> None:
        """Test that AtomicExpr cannot be instantiated directly."""
        try:
            AtomicExpr()  # type: ignore[abstract]
            raise AssertionError("Should not be able to instantiate AtomicExpr")
        except TypeError as e:
            assert "abstract" in str(e).lower()

    def test_field_based_expr_is_abstract(self) -> None:
        """Test that FieldBasedExpr cannot be instantiated directly."""
        try:
            FieldBasedExpr()  # type: ignore[abstract]
            raise AssertionError("Should not be able to instantiate FieldBasedExpr")
        except TypeError as e:
            assert "abstract" in str(e).lower()

    def test_single_valued_field_expr_is_abstract(self) -> None:
        """Test that SingleValuedFieldExpr cannot be instantiated directly."""
        try:
            SingleValuedFieldExpr()  # type: ignore[abstract]
            raise AssertionError("Should not be able to instantiate SingleValuedFieldExpr")
        except TypeError as e:
            assert "abstract" in str(e).lower()

    def test_comparison_expr_is_abstract(self) -> None:
        """Test that ComparisonExpr cannot be instantiated directly."""
        try:
            ComparisonExpr()  # type: ignore[abstract]
            raise AssertionError("Should not be able to instantiate ComparisonExpr")
        except TypeError as e:
            assert "abstract" in str(e).lower()

    def test_multi_valued_field_expr_is_abstract(self) -> None:
        """Test that MultiValuedFieldExpr cannot be instantiated directly."""
        try:
            MultiValuedFieldExpr()  # type: ignore[abstract]
            raise AssertionError("Should not be able to instantiate MultiValuedFieldExpr")
        except TypeError as e:
            assert "abstract" in str(e).lower()

    def test_composite_expr_is_abstract(self) -> None:
        """Test that CompositeExpr cannot be instantiated directly."""
        try:
            CompositeExpr()  # type: ignore[abstract]
            raise AssertionError("Should not be able to instantiate CompositeExpr")
        except TypeError as e:
            assert "abstract" in str(e).lower()

    def test_decorator_expr_is_abstract(self) -> None:
        """Test that DecoratorExpr cannot be instantiated directly."""
        try:
            DecoratorExpr()  # type: ignore[abstract]
            raise AssertionError("Should not be able to instantiate DecoratorExpr")
        except TypeError as e:
            assert "abstract" in str(e).lower()



"""Tests for MongoDB sort specification visitor."""

import pytest

from therismos.sorting import SortCriterion, SortOrder, SortSpec
from therismos.sorting.visitors.mongo import MongoVisitor


class TestMongoVisitorBasic:
    """Tests for basic MongoDB visitor functionality."""

    def test_single_ascending_criterion(self):
        """Test MongoDB format for single ascending criterion."""
        spec = SortSpec(
            [
                SortCriterion("age", SortOrder.ASCENDING),
            ]
        )
        visitor = MongoVisitor()

        result = spec.accept(visitor)

        assert result == {"age": 1}

    def test_single_descending_criterion(self):
        """Test MongoDB format for single descending criterion."""
        spec = SortSpec(
            [
                SortCriterion("name", SortOrder.DESCENDING),
            ]
        )
        visitor = MongoVisitor()

        result = spec.accept(visitor)

        assert result == {"name": -1}

    def test_multiple_criteria(self):
        """Test MongoDB format for multiple criteria."""
        spec = SortSpec(
            [
                SortCriterion("age", SortOrder.ASCENDING),
                SortCriterion("name", SortOrder.DESCENDING),
                SortCriterion("score", SortOrder.ASCENDING),
            ]
        )
        visitor = MongoVisitor()

        result = spec.accept(visitor)

        assert result == {"age": 1, "name": -1, "score": 1}

    def test_empty_spec(self):
        """Test MongoDB format for empty spec."""
        spec = SortSpec()
        visitor = MongoVisitor()

        result = spec.accept(visitor)

        assert result == {}

    def test_result_is_dict(self):
        """Test that result is a dictionary."""
        spec = SortSpec(
            [
                SortCriterion("x", SortOrder.ASCENDING),
            ]
        )
        visitor = MongoVisitor()

        result = spec.accept(visitor)

        assert isinstance(result, dict)


class TestMongoVisitorNoneOrder:
    """Tests for handling SortOrder.NONE."""

    def test_single_none_criterion_skipped(self):
        """Test that NONE order is skipped in MongoDB output."""
        spec = SortSpec(
            [
                SortCriterion("age", SortOrder.NONE),
            ]
        )
        visitor = MongoVisitor()

        result = spec.accept(visitor)

        assert result == {}

    def test_none_mixed_with_other_orders(self):
        """Test that NONE orders are skipped while others are preserved."""
        spec = SortSpec(
            [
                SortCriterion("age", SortOrder.ASCENDING),
                SortCriterion("name", SortOrder.NONE),
                SortCriterion("score", SortOrder.DESCENDING),
            ]
        )
        visitor = MongoVisitor()

        result = spec.accept(visitor)

        assert result == {"age": 1, "score": -1}
        assert "name" not in result

    def test_all_none_orders(self):
        """Test spec with all NONE orders results in empty dict."""
        spec = SortSpec(
            [
                SortCriterion("a", SortOrder.NONE),
                SortCriterion("b", SortOrder.NONE),
            ]
        )
        visitor = MongoVisitor()

        result = spec.accept(visitor)

        assert result == {}


class TestMongoVisitorDuplicateFields:
    """Tests for handling duplicate field names."""

    def test_duplicate_field_last_wins(self):
        """Test that when a field appears multiple times, last occurrence wins."""
        spec = SortSpec(
            [
                SortCriterion("age", SortOrder.ASCENDING),
                SortCriterion("name", SortOrder.DESCENDING),
                SortCriterion("age", SortOrder.DESCENDING),
            ]
        )
        visitor = MongoVisitor()

        result = spec.accept(visitor)

        # Last occurrence of "age" should be DESCENDING (-1)
        assert result == {"age": -1, "name": -1}

    def test_multiple_duplicates(self):
        """Test multiple duplicate fields."""
        spec = SortSpec(
            [
                SortCriterion("a", SortOrder.ASCENDING),
                SortCriterion("b", SortOrder.DESCENDING),
                SortCriterion("a", SortOrder.DESCENDING),
                SortCriterion("b", SortOrder.ASCENDING),
            ]
        )
        visitor = MongoVisitor()

        result = spec.accept(visitor)

        assert result == {"a": -1, "b": 1}

    def test_triple_occurrence(self):
        """Test field occurring three times."""
        spec = SortSpec(
            [
                SortCriterion("x", SortOrder.ASCENDING),
                SortCriterion("x", SortOrder.DESCENDING),
                SortCriterion("x", SortOrder.ASCENDING),
            ]
        )
        visitor = MongoVisitor()

        result = spec.accept(visitor)

        # Last occurrence is ASCENDING (1)
        assert result == {"x": 1}


class TestMongoVisitorDictOrdering:
    """Tests for dictionary ordering in results."""

    def test_dict_preserves_insertion_order(self):
        """Test that result dict preserves field order (Python 3.7+)."""
        spec = SortSpec(
            [
                SortCriterion("z", SortOrder.ASCENDING),
                SortCriterion("a", SortOrder.DESCENDING),
                SortCriterion("m", SortOrder.ASCENDING),
            ]
        )
        visitor = MongoVisitor()

        result = spec.accept(visitor)

        # Python 3.7+ dicts preserve insertion order
        keys = list(result.keys())
        assert keys == ["z", "a", "m"]


@pytest.mark.parametrize(
    "criteria,expected_mongo",
    [
        # Single criteria
        (
            [SortCriterion("x", SortOrder.ASCENDING)],
            {"x": 1},
        ),
        (
            [SortCriterion("y", SortOrder.DESCENDING)],
            {"y": -1},
        ),
        # Multiple criteria
        (
            [
                SortCriterion("a", SortOrder.ASCENDING),
                SortCriterion("b", SortOrder.DESCENDING),
            ],
            {"a": 1, "b": -1},
        ),
        # With NONE
        (
            [
                SortCriterion("a", SortOrder.ASCENDING),
                SortCriterion("b", SortOrder.NONE),
            ],
            {"a": 1},
        ),
        # Empty
        ([], {}),
        # Duplicates
        (
            [
                SortCriterion("x", SortOrder.ASCENDING),
                SortCriterion("x", SortOrder.DESCENDING),
            ],
            {"x": -1},
        ),
    ],
)
def test_mongo_visitor_parametrized(criteria, expected_mongo):
    """Parametrized test for MongoDB visitor with various specs."""
    spec = SortSpec(criteria)
    visitor = MongoVisitor()

    result = spec.accept(visitor)

    assert result == expected_mongo


class TestMongoVisitorIntegration:
    """Integration tests for MongoDB visitor."""

    def test_realistic_sorting_spec(self):
        """Test realistic sorting specification."""
        spec = SortSpec(
            [
                SortCriterion("created_at", SortOrder.DESCENDING),
                SortCriterion("priority", SortOrder.ASCENDING),
                SortCriterion("name", SortOrder.ASCENDING),
            ]
        )
        visitor = MongoVisitor()

        result = spec.accept(visitor)

        assert result == {"created_at": -1, "priority": 1, "name": 1}

    def test_mongo_dict_can_be_used_with_list_items(self):
        """Test that result can be converted to list of tuples for MongoDB."""
        spec = SortSpec(
            [
                SortCriterion("age", SortOrder.ASCENDING),
                SortCriterion("name", SortOrder.DESCENDING),
            ]
        )
        visitor = MongoVisitor()

        result = spec.accept(visitor)
        # MongoDB sort() accepts list of tuples
        sort_list = list(result.items())

        assert sort_list == [("age", 1), ("name", -1)]



"""Tests for basic expression creation and DSL."""

import re

import pytest

from therismos import (
    FALSE,
    TRUE,
    AllExpr,
    AnyExpr,
    Eq,
    F,
    Field,
    Ge,
    Gt,
    In,
    IsNull,
    Le,
    Lt,
    Ne,
    NotExpr,
    Regex,
)


class TestFieldCreation:
    """Test Field creation and basic operations."""

    def test_field_creation_simple(self) -> None:
        """Test creating a field without a type."""
        field = Field("name")
        assert field.name == "name"
        assert field.type_ is None

    def test_field_creation_with_type(self) -> None:
        """Test creating a field with a type."""
        field = Field("age", int)
        assert field.name == "age"
        assert field.type_ is int

    def test_f_helper_function(self) -> None:
        """Test the F() helper function."""
        field = F("email")
        assert field.name == "email"
        assert field.type_ is None

        field_with_type = F("count", int)
        assert field_with_type.name == "count"
        assert field_with_type.type_ is int


class TestAtomicExpressions:
    """Test atomic expression creation via DSL."""

    @pytest.mark.parametrize(
        "operation,value,expected_type",
        [
            (lambda f: f == 10, 10, Eq),
            (lambda f: f != 10, 10, Ne),
            (lambda f: f < 10, 10, Lt),
            (lambda f: f <= 10, 10, Le),
            (lambda f: f > 10, 10, Gt),
            (lambda f: f >= 10, 10, Ge),
        ],
    )
    def test_comparison_operators(self, operation: object, value: int, expected_type: type) -> None:
        """Test comparison operators create correct expression types."""
        field = F("age")
        expr = operation(field)
        assert isinstance(expr, expected_type)
        assert expr.field.name == "age"  # type: ignore
        assert expr.value == value  # type: ignore

    def test_regex_matches(self) -> None:
        """Test regex matching expression."""
        field = F("name")
        expr = field.matches(r"^A.*")
        assert isinstance(expr, Regex)
        assert expr.field.name == "name"
        assert expr.value == r"^A.*"
        assert expr.flags is None

    def test_regex_matches_with_flags(self) -> None:
        """Test regex matching with flags."""
        field = F("name")
        expr = field.matches(r"^A.*", re.IGNORECASE)
        assert isinstance(expr, Regex)
        assert expr.flags == re.IGNORECASE

    def test_in_with_values(self) -> None:
        """Test IN expression with individual values."""
        field = F("status")
        expr = field.is_in("active", "pending")
        assert isinstance(expr, In)
        assert expr.field.name == "status"
        assert expr.values == ("active", "pending")

    def test_one_of_with_list(self) -> None:
        """Test is_one_of with a list."""
        field = F("status")
        expr = field.is_one_of(["active", "pending", "completed"])
        assert isinstance(expr, In)
        assert expr.values == ("active", "pending", "completed")

    def test_is_null(self) -> None:
        """Test IS NULL expression."""
        field = F("deleted_at")
        expr = field.is_null()
        assert isinstance(expr, IsNull)
        assert expr.field.name == "deleted_at"
        assert expr.is_null is True

    def test_is_not_null(self) -> None:
        """Test IS NOT NULL expression."""
        field = F("created_at")
        expr = field.is_not_null()
        assert isinstance(expr, IsNull)
        assert expr.field.name == "created_at"
        assert expr.is_null is False


class TestCompoundExpressions:
    """Test compound expression creation via DSL."""

    def test_and_operator(self) -> None:
        """Test & operator creates AllExpr."""
        age = F("age")
        name = F("name")
        expr = (age > 18) & (name == "Alice")
        assert isinstance(expr, AllExpr)
        assert len(expr.exprs) == 2

    def test_or_operator(self) -> None:
        """Test | operator creates AnyExpr."""
        status = F("status")
        expr = (status == "active") | (status == "pending")
        assert isinstance(expr, AnyExpr)
        assert len(expr.exprs) == 2

    def test_not_operator(self) -> None:
        """Test ~ operator creates NotExpr."""
        age = F("age")
        expr = ~(age > 18)
        assert isinstance(expr, NotExpr)
        assert isinstance(expr.expr, Gt)

    def test_complex_expression(self) -> None:
        """Test complex nested expression."""
        age = F("age")
        name = F("name")
        status = F("status")

        expr = ((age > 18) & (name == "Alice")) | (status == "admin")
        assert isinstance(expr, AnyExpr)
        assert isinstance(expr.exprs[0], AllExpr)


class TestImmutability:
    """Test that all expressions are immutable."""

    def test_field_immutability(self) -> None:
        """Test that Field is frozen."""
        field = F("name")
        with pytest.raises(AttributeError):
            field.name = "other"  # type: ignore

    def test_eq_immutability(self) -> None:
        """Test that Eq is frozen."""
        field = F("age")
        expr = field == 10
        with pytest.raises(AttributeError):
            expr.value = 20  # type: ignore

    def test_all_expr_immutability(self) -> None:
        """Test that AllExpr is frozen."""
        age = F("age")
        expr = (age > 18) & (age < 65)
        with pytest.raises(AttributeError):
            expr.exprs = ()  # type: ignore


class TestLogicalConstants:
    """Test TRUE and FALSE constants."""

    def test_true_constant(self) -> None:
        """Test TRUE constant."""
        from therismos import TrueExpr

        assert isinstance(TRUE, TrueExpr)

    def test_false_constant(self) -> None:
        """Test FALSE constant."""
        from therismos import FalseExpr

        assert isinstance(FALSE, FalseExpr)

    def test_true_false_in_expressions(self) -> None:
        """Test using TRUE/FALSE in compound expressions."""
        age = F("age")
        expr1 = (age > 18) & TRUE
        assert isinstance(expr1, AllExpr)

        expr2 = (age > 18) | FALSE
        assert isinstance(expr2, AnyExpr)



"""Tests for grouping specification visitors."""

import pytest

from therismos.grouping import Aggregation, AggregationFunction, GroupSpec
from therismos.grouping.visitors import DictVisitor, FieldGathererVisitor, StringVisitor
from therismos.grouping.visitors.mongo import MongoVisitor


class TestStringVisitor:
    """Tests for StringVisitor."""

    def test_simple_group_spec(self):
        """Test converting a simple GroupSpec to string."""
        spec = GroupSpec(
            group_by=["category"],
            aggregations=[Aggregation("total", AggregationFunction.COUNT)],
        )

        visitor = StringVisitor()
        result = spec.accept(visitor)

        assert result == '("category", "total:count")'

    def test_multiple_fields_and_aggregations(self):
        """Test converting GroupSpec with multiple fields and aggregations."""
        spec = GroupSpec(
            group_by=["category", "region"],
            aggregations=[
                Aggregation("total", AggregationFunction.COUNT),
                Aggregation("min_price", AggregationFunction.MIN, "price"),
                Aggregation("avg_price", AggregationFunction.AVERAGE, "price"),
            ],
        )

        visitor = StringVisitor()
        result = spec.accept(visitor)

        expected = '("category,region", "total:count,min_price:min:price,avg_price:average:price")'
        assert result == expected

    def test_empty_group_by(self):
        """Test converting GroupSpec with no grouping fields."""
        spec = GroupSpec(
            group_by=[],
            aggregations=[Aggregation("total", AggregationFunction.COUNT)],
        )

        visitor = StringVisitor()
        result = spec.accept(visitor)

        assert result == '("", "total:count")'

    def test_empty_aggregations(self):
        """Test converting GroupSpec with no aggregations."""
        spec = GroupSpec(
            group_by=["category"],
            aggregations=[],
        )

        visitor = StringVisitor()
        result = spec.accept(visitor)

        assert result == '("category", "")'


class TestDictVisitor:
    """Tests for DictVisitor."""

    def test_simple_group_spec(self):
        """Test converting a simple GroupSpec to dictionary."""
        spec = GroupSpec(
            group_by=["category"],
            aggregations=[Aggregation("total", AggregationFunction.COUNT)],
        )

        visitor = DictVisitor()
        result = spec.accept(visitor)

        assert result == {
            "group_by": ["category"],
            "aggregations": [
                {"id": "total", "function": "count", "field": None},
            ],
        }

    def test_multiple_fields_and_aggregations(self):
        """Test converting GroupSpec with multiple fields and aggregations."""
        spec = GroupSpec(
            group_by=["category", "region"],
            aggregations=[
                Aggregation("total", AggregationFunction.COUNT),
                Aggregation("min_price", AggregationFunction.MIN, "price"),
                Aggregation("max_price", AggregationFunction.MAX, "price"),
            ],
        )

        visitor = DictVisitor()
        result = spec.accept(visitor)

        assert result == {
            "group_by": ["category", "region"],
            "aggregations": [
                {"id": "total", "function": "count", "field": None},
                {"id": "min_price", "function": "min", "field": "price"},
                {"id": "max_price", "function": "max", "field": "price"},
            ],
        }

    def test_percentile_aggregations(self):
        """Test converting GroupSpec with percentile aggregations."""
        spec = GroupSpec(
            group_by=["service"],
            aggregations=[
                Aggregation("p95", AggregationFunction.P95, "latency"),
                Aggregation("p99", AggregationFunction.P99, "latency"),
            ],
        )

        visitor = DictVisitor()
        result = spec.accept(visitor)

        assert result["aggregations"] == [
            {"id": "p95", "function": "p95", "field": "latency"},
            {"id": "p99", "function": "p99", "field": "latency"},
        ]


class TestFieldGathererVisitor:
    """Tests for FieldGathererVisitor."""

    def test_simple_group_spec(self):
        """Test gathering fields from a simple GroupSpec."""
        spec = GroupSpec(
            group_by=["category"],
            aggregations=[Aggregation("total", AggregationFunction.COUNT)],
        )

        visitor = FieldGathererVisitor()
        spec.accept(visitor)

        assert visitor.field_names == {"category"}

    def test_multiple_fields_and_aggregations(self):
        """Test gathering fields from GroupSpec with multiple fields."""
        spec = GroupSpec(
            group_by=["category", "region"],
            aggregations=[
                Aggregation("total", AggregationFunction.COUNT),
                Aggregation("min_price", AggregationFunction.MIN, "price"),
                Aggregation("max_revenue", AggregationFunction.MAX, "revenue"),
            ],
        )

        visitor = FieldGathererVisitor()
        spec.accept(visitor)

        assert visitor.field_names == {"category", "region", "price", "revenue"}

    def test_duplicate_fields(self):
        """Test that duplicate field names are collected only once."""
        spec = GroupSpec(
            group_by=["category", "region", "category"],
            aggregations=[
                Aggregation("min_price", AggregationFunction.MIN, "price"),
                Aggregation("max_price", AggregationFunction.MAX, "price"),
            ],
        )

        visitor = FieldGathererVisitor()
        spec.accept(visitor)

        # Set should contain unique values
        assert visitor.field_names == {"category", "region", "price"}

    def test_empty_group_by(self):
        """Test gathering fields with no grouping fields."""
        spec = GroupSpec(
            group_by=[],
            aggregations=[
                Aggregation("total", AggregationFunction.COUNT),
                Aggregation("avg_price", AggregationFunction.AVERAGE, "price"),
            ],
        )

        visitor = FieldGathererVisitor()
        spec.accept(visitor)

        # Only aggregation field
        assert visitor.field_names == {"price"}


class TestMongoVisitor:
    """Tests for MongoVisitor."""

    def test_simple_count_aggregation(self):
        """Test converting simple COUNT aggregation to MongoDB format."""
        spec = GroupSpec(
            group_by=["category"],
            aggregations=[Aggregation("total", AggregationFunction.COUNT)],
        )

        visitor = MongoVisitor()
        result = spec.accept(visitor)

        assert result == {
            "$group": {
                "_id": "$category",
                "total": {"$sum": 1},
            }
        }

    def test_multiple_grouping_fields(self):
        """Test MongoDB format with multiple grouping fields."""
        spec = GroupSpec(
            group_by=["category", "region"],
            aggregations=[Aggregation("total", AggregationFunction.COUNT)],
        )

        visitor = MongoVisitor()
        result = spec.accept(visitor)

        assert result == {
            "$group": {
                "_id": {
                    "category": "$category",
                    "region": "$region",
                },
                "total": {"$sum": 1},
            }
        }

    def test_various_aggregation_functions(self):
        """Test various aggregation functions in MongoDB format."""
        spec = GroupSpec(
            group_by=["category"],
            aggregations=[
                Aggregation("count", AggregationFunction.COUNT),
                Aggregation("total_revenue", AggregationFunction.SUM, "revenue"),
                Aggregation("min_price", AggregationFunction.MIN, "price"),
                Aggregation("max_price", AggregationFunction.MAX, "price"),
                Aggregation("avg_price", AggregationFunction.AVERAGE, "price"),
                Aggregation("stddev_price", AggregationFunction.STDDEV, "price"),
            ],
        )

        visitor = MongoVisitor()
        result = spec.accept(visitor)

        assert result["$group"]["count"] == {"$sum": 1}
        assert result["$group"]["total_revenue"] == {"$sum": "$revenue"}
        assert result["$group"]["min_price"] == {"$min": "$price"}
        assert result["$group"]["max_price"] == {"$max": "$price"}
        assert result["$group"]["avg_price"] == {"$avg": "$price"}
        assert result["$group"]["stddev_price"] == {"$stdDevPop": "$price"}

    def test_median_aggregation(self):
        """Test MEDIAN aggregation in MongoDB format."""
        spec = GroupSpec(
            group_by=["category"],
            aggregations=[Aggregation("median_price", AggregationFunction.MEDIAN, "price")],
        )

        visitor = MongoVisitor()
        result = spec.accept(visitor)

        assert result["$group"]["median_price"] == {
            "$median": {
                "input": "$price",
                "method": "approximate",
            }
        }

    def test_quartile_aggregations(self):
        """Test Q1 and Q3 aggregations in MongoDB format."""
        spec = GroupSpec(
            group_by=["category"],
            aggregations=[
                Aggregation("q1", AggregationFunction.Q1, "salary"),
                Aggregation("q3", AggregationFunction.Q3, "salary"),
            ],
        )

        visitor = MongoVisitor()
        result = spec.accept(visitor)

        assert result["$group"]["q1"] == {
            "$percentile": {
                "input": "$salary",
                "p": [0.25],
                "method": "approximate",
            }
        }
        assert result["$group"]["q3"] == {
            "$percentile": {
                "input": "$salary",
                "p": [0.75],
                "method": "approximate",
            }
        }

    @pytest.mark.parametrize(
        "func,percentile",
        [
            (AggregationFunction.P01, 0.01),
            (AggregationFunction.P05, 0.05),
            (AggregationFunction.P10, 0.10),
            (AggregationFunction.P90, 0.90),
            (AggregationFunction.P95, 0.95),
            (AggregationFunction.P99, 0.99),
        ],
    )
    def test_percentile_aggregations(self, func, percentile):
        """Test various percentile aggregations in MongoDB format."""
        spec = GroupSpec(
            group_by=["category"],
            aggregations=[Aggregation("pct", func, "latency")],
        )

        visitor = MongoVisitor()
        result = spec.accept(visitor)

        assert result["$group"]["pct"] == {
            "$percentile": {
                "input": "$latency",
                "p": [percentile],
                "method": "approximate",
            }
        }

    def test_no_grouping_fields(self):
        """Test MongoDB format with no grouping fields (global aggregation)."""
        spec = GroupSpec(
            group_by=[],
            aggregations=[Aggregation("total", AggregationFunction.COUNT)],
        )

        visitor = MongoVisitor()
        result = spec.accept(visitor)

        assert result == {
            "$group": {
                "_id": None,
                "total": {"$sum": 1},
            }
        }

    def test_simplify_single_group_disabled(self):
        """Test with simplify_single_group=False."""
        spec = GroupSpec(
            group_by=["category"],
            aggregations=[Aggregation("total", AggregationFunction.COUNT)],
        )

        visitor = MongoVisitor(simplify_single_group=False)
        result = spec.accept(visitor)

        # Should use composite format even for single field
        assert result == {
            "$group": {
                "_id": {"category": "$category"},
                "total": {"$sum": 1},
            }
        }



"""MongoDB aggregation pipeline visitor for therismos grouping.

This module provides a visitor that converts therismos grouping specifications into
MongoDB aggregation pipeline $group stages compatible with PyMongo and Motor.

The MongoDB visitor does not require PyMongo or Motor to be installed for the
conversion itself - it simply returns dictionaries in MongoDB $group format.
However, you'll need one of these libraries to actually execute the queries:

- PyMongo: Synchronous MongoDB driver
- Motor: Asynchronous MongoDB driver for asyncio

Example
-------
>>> from therismos.grouping import GroupSpec, Aggregation, AggregationFunction
>>> from therismos.grouping.visitors.mongo import MongoVisitor
>>>
>>> spec = GroupSpec(
...     group_by=["category", "region"],
...     aggregations=[
...         Aggregation("total", AggregationFunction.COUNT),
...         Aggregation("min_price", AggregationFunction.MIN, "price"),
...         Aggregation("avg_price", AggregationFunction.AVERAGE, "price"),
...     ],
... )
>>>
>>> visitor = MongoVisitor()
>>> group_stage = spec.accept(visitor)
>>> # Result: {
>>> #     "$group": {
>>> #         "_id": {"category": "$category", "region": "$region"},
>>> #         "total": {"$sum": 1},
>>> #         "min_price": {"$min": "$price"},
>>> #         "avg_price": {"$avg": "$price"}
>>> #     }
>>> # }
>>>
>>> # Use with PyMongo
>>> # pipeline = [group_stage]
>>> # results = collection.aggregate(pipeline)
>>>
>>> # Use with Motor
>>> # pipeline = [group_stage]
>>> # results = await collection.aggregate(pipeline).to_list(length=None)

Notes
-----
- COUNT aggregation converts to {"$sum": 1}
- SUM aggregation converts to {"$sum": "$field"}
- MIN/MAX convert to {"$min": "$field"} / {"$max": "$field"}
- AVERAGE converts to {"$avg": "$field"}
- STDDEV converts to {"$stdDevPop": "$field"}
- MEDIAN and percentiles use $percentile operator (MongoDB 7.0+)
- The _id field contains the grouping fields as a composite key
- For single grouping field, _id can be simplified to "$field"
"""

from __future__ import annotations

from typing import Any

from therismos.grouping._grouping import Aggregation, AggregationFunction, GroupSpec


class MongoVisitor:
    """Visitor that converts grouping specifications to MongoDB $group stages.

    This visitor traverses grouping specifications and generates MongoDB
    aggregation pipeline $group stages that can be used with PyMongo's
    aggregate() or Motor's aggregate() methods.

    :ivar simplify_single_group: If True, simplify _id to "$field" for single grouping field.
    :vartype simplify_single_group: bool

    Example
    -------
    >>> from therismos.grouping import GroupSpec, Aggregation, AggregationFunction
    >>> from therismos.grouping.visitors.mongo import MongoVisitor
    >>>
    >>> spec = GroupSpec(
    ...     group_by=["status"],
    ...     aggregations=[
    ...         Aggregation("count", AggregationFunction.COUNT),
    ...         Aggregation("avg_age", AggregationFunction.AVERAGE, "age"),
    ...     ],
    ... )
    >>>
    >>> visitor = MongoVisitor(simplify_single_group=True)
    >>> group_stage = spec.accept(visitor)
    >>> print(group_stage)
    {
        "$group": {
            "_id": "$status",
            "count": {"$sum": 1},
            "avg_age": {"$avg": "$age"}
        }
    }
    """

    def __init__(self, simplify_single_group: bool = True) -> None:
        """Initialize the MongoDB visitor.

        :param simplify_single_group: If True, use "$field" for single grouping field
            instead of {"field": "$field"}.
        :type simplify_single_group: bool
        """
        self.simplify_single_group = simplify_single_group

    def _convert_aggregation(self, agg: Aggregation) -> dict[str, Any]:
        """Convert an aggregation to MongoDB accumulator format.

        :param agg: The aggregation to convert.
        :type agg: Aggregation
        :returns: MongoDB accumulator expression.
        :rtype: dict[str, Any]
        :raises ValueError: If the aggregation function is not supported.
        """
        func = agg.function
        field_ref = f"${agg.field}" if agg.field else None

        if func == AggregationFunction.COUNT:
            return {"$sum": 1}
        elif func == AggregationFunction.SUM:
            return {"$sum": field_ref}
        elif func == AggregationFunction.MIN:
            return {"$min": field_ref}
        elif func == AggregationFunction.MAX:
            return {"$max": field_ref}
        elif func == AggregationFunction.AVERAGE:
            return {"$avg": field_ref}
        elif func == AggregationFunction.STDDEV:
            # Use population standard deviation (sample stddev would be $stdDevSamp)
            return {"$stdDevPop": field_ref}
        elif func == AggregationFunction.MEDIAN:
            # MongoDB 7.0+ percentile operator
            return {
                "$median": {
                    "input": field_ref,
                    "method": "approximate",
                }
            }
        elif func == AggregationFunction.Q1:
            # First quartile = 25th percentile
            return {
                "$percentile": {
                    "input": field_ref,
                    "p": [0.25],
                    "method": "approximate",
                }
            }
        elif func == AggregationFunction.Q3:
            # Third quartile = 75th percentile
            return {
                "$percentile": {
                    "input": field_ref,
                    "p": [0.75],
                    "method": "approximate",
                }
            }
        elif func == AggregationFunction.P01:
            return {
                "$percentile": {
                    "input": field_ref,
                    "p": [0.01],
                    "method": "approximate",
                }
            }
        elif func == AggregationFunction.P05:
            return {
                "$percentile": {
                    "input": field_ref,
                    "p": [0.05],
                    "method": "approximate",
                }
            }
        elif func == AggregationFunction.P10:
            return {
                "$percentile": {
                    "input": field_ref,
                    "p": [0.10],
                    "method": "approximate",
                }
            }
        elif func == AggregationFunction.P90:
            return {
                "$percentile": {
                    "input": field_ref,
                    "p": [0.90],
                    "method": "approximate",
                }
            }
        elif func == AggregationFunction.P95:
            return {
                "$percentile": {
                    "input": field_ref,
                    "p": [0.95],
                    "method": "approximate",
                }
            }
        elif func == AggregationFunction.P99:
            return {
                "$percentile": {
                    "input": field_ref,
                    "p": [0.99],
                    "method": "approximate",
                }
            }
        else:
            msg = f"Unsupported aggregation function: {func}"
            raise ValueError(msg)

    def visit_group_spec(self, spec: GroupSpec) -> dict[str, Any]:
        """Convert a grouping specification to MongoDB $group stage.

        :param spec: The grouping specification to visit.
        :type spec: GroupSpec
        :returns: MongoDB aggregation pipeline $group stage.
        :rtype: dict[str, Any]
        """
        # Build the _id field for grouping
        if len(spec.group_by) == 0:
            # No grouping fields means group all documents together
            group_id: Any = None
        elif len(spec.group_by) == 1 and self.simplify_single_group:
            # Single field: use simple "$field" format
            group_id = f"${spec.group_by[0]}"
        else:
            # Multiple fields: use composite key format
            group_id = {field: f"${field}" for field in spec.group_by}

        # Build the accumulator fields
        accumulators = {
            agg.id: self._convert_aggregation(agg) for agg in spec.aggregations.values()
        }

        # Construct the $group stage
        return {
            "$group": {
                "_id": group_id,
                **accumulators,
            }
        }


__all__ = [
    "MongoVisitor",
]



"""Tests for multi-valued field evaluation."""

from therismos.expr import F, unwind_data


class TestEqMultiValuedEval:
    """Test Eq evaluation with list-valued fields."""

    def test_eq_list_any_match(self) -> None:
        """Test that Eq returns True if ANY value matches."""
        expr = F("tags") == "python"
        assert expr.evaluate(unwind_data({"tags": ["python", "java", "rust"]})) is True

    def test_eq_list_no_match(self) -> None:
        """Test that Eq returns False if no values match."""
        expr = F("tags") == "python"
        assert expr.evaluate(unwind_data({"tags": ["java", "rust", "go"]})) is False

    def test_eq_single_value_backward_compat(self) -> None:
        """Test that Eq works with single values (backward compatibility)."""
        expr = F("name") == "alice"
        assert expr.evaluate(unwind_data({"name": "alice"})) is True
        assert expr.evaluate(unwind_data({"name": "bob"})) is False


class TestNeMultiValuedEval:
    """Test Ne evaluation with list-valued fields."""

    def test_ne_list_none_match(self) -> None:
        """Test that Ne returns True if NONE of the values match."""
        expr = F("tags") != "python"
        assert expr.evaluate(unwind_data({"tags": ["java", "rust", "go"]})) is True

    def test_ne_list_some_match(self) -> None:
        """Test that Ne returns False if any value matches."""
        expr = F("tags") != "python"
        assert expr.evaluate(unwind_data({"tags": ["python", "java", "rust"]})) is False


class TestComparisonMultiValuedEval:
    """Test comparison expressions with list-valued fields."""

    def test_lt_list_any_less(self) -> None:
        """Test that Lt returns True if ANY value is less than target."""
        expr = F("scores") < 50
        assert expr.evaluate(unwind_data({"scores": [30, 60, 80]})) is True

    def test_gt_list_any_greater(self) -> None:
        """Test that Gt returns True if ANY value is greater than target."""
        expr = F("scores") > 50
        assert expr.evaluate(unwind_data({"scores": [30, 40, 60]})) is True


class TestRegexMultiValuedEval:
    """Test Regex evaluation with list-valued fields."""

    def test_regex_list_any_match(self) -> None:
        """Test that Regex returns True if ANY value matches."""
        expr = F("tags").matches(r"py.*")
        assert expr.evaluate(unwind_data({"tags": ["python", "java", "rust"]})) is True

    def test_regex_list_no_match(self) -> None:
        """Test that Regex returns False if no values match."""
        expr = F("tags").matches(r"py.*")
        assert expr.evaluate(unwind_data({"tags": ["java", "rust", "go"]})) is False


class TestIsNullMultiValuedEval:
    """Test IsNull evaluation with list-valued fields."""

    def test_is_null_empty_list(self) -> None:
        """Test that empty list is treated as null."""
        expr = F("tags").is_null()
        assert expr.evaluate(unwind_data({"tags": []})) is True

    def test_is_null_list_with_none_values(self) -> None:
        """Test that list with all None values satisfies is_null."""
        expr = F("tags").is_null()
        assert expr.evaluate(unwind_data({"tags": [None, None]})) is True

    def test_is_null_list_with_some_none(self) -> None:
        """Test that list with some None values does not satisfy is_null."""
        expr = F("tags").is_null()
        assert expr.evaluate(unwind_data({"tags": [None, "python"]})) is False

    def test_is_not_null_list_with_all_non_none(self) -> None:
        """Test that list with all non-None values satisfies is_not_null."""
        expr = F("tags").is_not_null()
        assert expr.evaluate(unwind_data({"tags": ["python", "java"]})) is True



"""Serializer for converting sort specifications to/from compact string format.

This module provides serialization capabilities for converting sort specifications
to and from compact comma-separated string representations, suitable for use in
URL query parameters and APIs.

Format
------
Sort specifications are serialized as comma-separated lists of field criteria:
- Each criterion is a field name optionally prefixed with + (ascending) or - (descending)
- No prefix defaults to ascending

Examples
--------
>>> from therismos.sorting import SortSpec, SortCriterion, SortOrder
>>> from therismos.sorting.serializer import Serializer
>>>
>>> # Basic serialization
>>> spec = SortSpec([
...     SortCriterion("age", SortOrder.ASCENDING),
...     SortCriterion("created_at", SortOrder.DESCENDING),
... ])
>>> serializer = Serializer()
>>> text = serializer.serialize(spec)
>>> # Result: "age,-created_at"
>>>
>>> # Deserialization
>>> spec = serializer.deserialize("name,-score,+priority")
>>> # Result: SortSpec with name ASC, score DESC, priority ASC
"""

from __future__ import annotations

from therismos.sorting._sorting import SortCriterion, SortOrder, SortSpec


class Serializer:
    """Serializer for converting sort specifications to/from string format.

    Example:
        >>> serializer = Serializer()
        >>> spec = SortSpec([
        ...     SortCriterion("age", SortOrder.ASCENDING),
        ...     SortCriterion("name", SortOrder.DESCENDING),
        ... ])
        >>> text = serializer.serialize(spec)
        >>> # Result: "age,-name"
    """

    def serialize(self, spec: SortSpec) -> str:
        """Serialize a sort specification to string format.

        :param spec: The sort specification to serialize.
        :type spec: SortSpec
        :returns: Serialized string representation.
        :rtype: str

        Example:
            >>> spec = SortSpec([
            ...     SortCriterion("age", SortOrder.ASCENDING),
            ...     SortCriterion("name", SortOrder.DESCENDING),
            ... ])
            >>> serializer.serialize(spec)
            'age,-name'
        """
        parts = []

        for criterion in spec:
            # Skip NONE orders during serialization
            if criterion.order == SortOrder.NONE:
                continue

            # Build the field representation
            field_str = criterion.field

            # Add order prefix
            if criterion.order == SortOrder.DESCENDING:
                field_str = "-" + field_str

            parts.append(field_str)

        return ",".join(parts)

    def deserialize(self, text: str) -> SortSpec:
        """Deserialize a string to a sort specification.

        :param text: The serialized sort specification string.
        :type text: str
        :returns: The deserialized sort specification.
        :rtype: SortSpec
        :raises ValueError: If the string format is invalid.

        Example:
            >>> serializer.deserialize("age,-name,+score")
            SortSpec([...])  # age ASC, name DESC, score ASC
        """
        if not text or text.strip() == "":
            return SortSpec()

        criteria = []
        parts = text.split(",")

        for part in parts:
            part = part.strip()
            if not part:
                continue

            # Determine order based on prefix
            if part.startswith("-"):
                order = SortOrder.DESCENDING
                field_name = part[1:]
            elif part.startswith("+"):
                order = SortOrder.ASCENDING
                field_name = part[1:]
            else:
                order = SortOrder.ASCENDING
                field_name = part

            # Validate field name
            if not field_name:
                msg = "Field name cannot be empty"
                raise ValueError(msg)

            # Create the criterion
            criterion = SortCriterion(field_name, order)
            criteria.append(criterion)

        return SortSpec(criteria)


__all__ = [
    "Serializer",
]
